/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <fcntl.h>
#include <unistd.h>
#include <stropts.h>
#include <errno.h>

int
auth(int netfd)
{
	int afd;
	struct strrecvfd recv;

	/*
	 * Open a mounted stream pipe to the authenticator.
	 */
	if ((afd = open("/var/.authpipe", O_RDWR)) < 0) {
		close(netfd);
		return(-1);
	}

	/*
	 * Send the network connection to be authenticated.
	 */
	if (ioctl(afd, I_SENDFD, netfd) < 0) {
		close(afd);
		close(netfd);
		return(-1);
	}

	/*
	 * We don't need the network file descriptor.
	 * The authenticator will pass us back the
	 * file descriptor to use.
	 */
	close(netfd);
	if (ioctl(afd, I_RECVFD, &recv) < 0) {
		recv.fd = -1;
	} else if (recv.uid != 0) {	/* impostor */
		close(recv.fd);
		errno = EACCES;
		recv.fd = -1;
	}
	close(afd);
	return(recv.fd);
}
