/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <sys/stream.h>
#include <sys/tihdr.h>
#include <stropts.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>

/*
 * The T_info_ack structure defined prior to SVR4 contained
 * one long integer less than the current structure.
 */
#define OLDSIZE	(sizeof(struct T_info_ack)-sizeof(long))

char *errmsg =
  "tisz: Incorrect response from transport provider";

extern void fatal(const char *, ...);

void
main(int argc, char *argv[])
{
	int fd, ret;
	struct strbuf ctl;
	struct T_info_ack ack;
	struct T_info_req req;
	int flag = RS_HIPRI;

	if (argc != 2)
		fatal("usage: tisz device");
	if ((fd = open(argv[1], O_RDWR)) < 0)
		fatal("tisz: Cannot open transport provider");

	/*
	 * Create an information request transport primitive
	 * and send it to the transport provider.  The format
	 * is one M_PCPROTO message block with no data blocks.
	 */
	ctl.buf = (char *)&req;
	ctl.len = sizeof(struct T_info_req);
	req.PRIM_type = T_INFO_REQ;
	if (putmsg(fd, &ctl, NULL, RS_HIPRI) < 0)
		fatal("tisz: Cannot send request");

	/*
	 * Receive the information acknowledgement from
	 * the transport provider.  The format is one
	 * M_PCPROTO message block with no data blocks.
	 */
	ctl.buf = (char *)&ack;
	ctl.maxlen = sizeof(struct T_info_ack);
	if ((ret = getmsg(fd, &ctl, NULL, &flag)) != 0) {
		if (ret < 0)
			fatal("tisz: Cannot receive response");
		else
			fatal(errmsg);
	}

	/*
	 * Validate the message and print the sizes.
	 */
	if (((ctl.len != sizeof(struct T_info_ack)) &&
	  (ctl.len != OLDSIZE)) ||
	  (ack.PRIM_type != T_INFO_ACK))
		fatal(errmsg);
	printf("TIDU size = %d bytes\n", ack.TIDU_size);
	printf("TSDU size = %d bytes\n", ack.TSDU_size);
	exit(0);
}
