/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <tiuser.h>
#include <signal.h>

void timeout(int);
extern int t_errno;
extern char *t_errlist[];
extern void log(const char *, ...);

#define tlog(STR) log("%s: %s", (STR), t_errlist[t_errno])
#define CMDSZ	256
#define BUFSZ	1024

void
doservice(int fd)
{
	int i, n, flags;
	char *p;
	FILE *fp;
	struct sigaction sa;
	char cmd[CMDSZ];
	char buf[BUFSZ];

	/*
	 * Receive the command string.  Must make multiple
	 * calls to t_rcv because the original message
	 * might have been fragmented during transmission.
	 */
	n = t_rcv(fd, cmd, CMDSZ, &flags);
	if (n < 0) {
		tlog("Cannot receive request");
		exit(1);
	}

	/*
	 * p is the current location in the command string
	 * and n is the amount remaining to be read.
	 */
	p = &cmd[n];
	n = cmd[0] - n;
	while (n > 0) {
		i = t_rcv(fd, p, (int)(CMDSZ + p - cmd), &flags);
		if (i < 0) {
			tlog("Cannot receive request");
			exit(1);
		}
		p += i;
		n -= i;
	}

	/*
	 * Execute the command.  fp is open to the
	 * command's standard output.
	 */
	fp = popen(&cmd[1], "r");
	if (fp) {
		/*
		 * Read the output of the command and send it
		 * back to the client.
		 */
		while ((n = fread(buf, sizeof(char), BUFSZ,
		  fp)) != 0) {
			if ((i = t_snd(fd, buf, n, 0)) != n) {
				if (i < 0)
					tlog("Cannot send data");
				else
					log("Partial t_snd");
				exit(0);
			}
		}
		pclose(fp);

		/*
		 * Send the terminating message to the client.
		 */
		buf[0] = '\0';
		if (t_snd(fd, buf, 1, 0) < 0) {
			tlog("Cannot send data");
			exit(0);
		}
	} else {
		log("Cannot exec command");
		exit(0);
	}

	/*
	 * Set an alarm so we don't block forever in t_rcv,
	 * awaiting the disconnect or orderly release.
	 */
	sa.sa_handler = timeout;
	sa.sa_flags = 0;
	sigemptyset(&sa.sa_mask);
	sigaction(SIGALRM, &sa, NULL);
	alarm(60);
	t_rcv(fd, buf, 1, &flags);
	exit(0);
}

void
timeout(int sig)
{
	log("Never got disconnect");
	exit(0);
}
