/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/utsname.h>
#include <rpc/rpc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "runame.h"

extern void daemonize(const char *);
extern void log(const char *, ...);
static void uname_prog(struct svc_req *, SVCXPRT *);

void
main()
{
	/*
	 * Become a daemon.
	 */
	daemonize("runamed");

	/*
	 * Create the server handle(s).  svc_create will
	 * unregister UNAME_PROG and UNAME_VER_[12] if they
	 * are currently registered with the rpcbinder.
	 */
	if (svc_create(uname_prog, UNAME_PROG, UNAME_VER_1,
	  "visible") == 0) {
		log("runamed: cannot start service");
		exit(1);
	}
	if (svc_create(uname_prog, UNAME_PROG, UNAME_VER_2,
	  "visible") == 0) {
		log("runamed: cannot start service");
		rpcb_unset(UNAME_PROG, UNAME_VER_1, NULL);
		exit(1);
	}

	/*
	 * Wait for requests and service them.
	 */
	svc_run();

	/*
	 * svc_run should never return.
	 */
	log("error: unexpected return from svc_run");

	/*
	 * Unregister this program and versions
	 * before exiting.
	 */
	rpcb_unset(UNAME_PROG, UNAME_VER_1, NULL);
	rpcb_unset(UNAME_PROG, UNAME_VER_2, NULL);
	exit(1);
}

static void
uname_prog(struct svc_req *req, SVCXPRT *handle)
{
	static struct utsname u;
	int ret;

	switch (req->rq_proc) {
	case NULLPROC:			/* null procedure */
		svc_sendreply(handle, xdr_void, NULL);
		return;

	case UNAME_PROC:			/* return utsname */
		break;

	default:				/* unknown procedure */
		svcerr_noproc(handle);
		return;
	}

	/*
	 * Copy dummy names to utsname structure elements.
	 * Then call uname.  If uname fails, we send dummy
	 * names back to the client.
	 */
	strcpy(u.sysname, "unknown");
	strcpy(u.nodename, "unknown");
	strcpy(u.release, "unknown");
	strcpy(u.version, "unknown");
	strcpy(u.machine, "unknown");
	uname(&u);


	/*
	 * Send reply to client.  If version 1, use
	 * xdr_utsname_1; else use xdr_utsname_2.
	 */
	if (req->rq_vers == UNAME_VER_1)
		ret = svc_sendreply(handle, xdr_utsname_1,
		  (char *)&u);
	else
		ret = svc_sendreply(handle, xdr_utsname_2,
		  (char *)&u);
	if (ret == 0)
		svcerr_systemerr(handle);
}
